package eu.mhsl.craftattack.spawn.appliances.tablist;

import eu.mhsl.craftattack.spawn.Main;
import eu.mhsl.craftattack.spawn.appliance.Appliance;
import eu.mhsl.craftattack.spawn.appliances.report.Report;
import eu.mhsl.craftattack.spawn.util.statistics.NetworkMonitor;
import eu.mhsl.craftattack.spawn.util.text.ComponentUtil;
import eu.mhsl.craftattack.spawn.util.IteratorUtil;
import eu.mhsl.craftattack.spawn.util.text.RainbowComponent;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import net.kyori.adventure.util.Ticks;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.bukkit.event.Listener;
import org.jetbrains.annotations.NotNull;

import java.time.Duration;
import java.util.List;


public class Tablist extends Appliance {
    private final int refreshRate = Ticks.TICKS_PER_SECOND * 3;
    private final RainbowComponent serverName = new RainbowComponent(" CraftAttack 7 ", 7, 3);
    private NetworkMonitor networkMonitor;

    public Tablist() {
        super("tablist");
    }

    @Override
    public void onEnable() {
        this.networkMonitor = new NetworkMonitor(localConfig().getString("interface"), Duration.ofSeconds(1));
        Bukkit.getScheduler().runTaskTimerAsynchronously(
                Main.instance(),
                () -> IteratorUtil.onlinePlayers(this::updateHeader),
                refreshRate,
                refreshRate
        );
    }

    @Override
    public void onDisable() {
        this.networkMonitor.stop();
    }

    public void fullUpdate(Player player) {
        updateHeader(player);
        updateFooter(player);
    }

    private void updateHeader(Player player) {
        player.sendPlayerListHeader(
                Component.newline()
                        .append(serverName.getRainbowState()).appendNewline()
                        .append(Component.text("mhsl.eu", NamedTextColor.GOLD)).appendNewline().appendNewline()
                        .append(ComponentUtil.getFormattedMSPT()).appendNewline().appendNewline()
                        .append(ComponentUtil.getFormattedPing(player)).appendNewline()
                        .append(ComponentUtil.getFormattedNetworkStats(
                            this.networkMonitor.getTraffic(),
                            this.networkMonitor.getPackets())
                        ).appendNewline()
        );
    }

    private void updateFooter(Player player) {
        player.sendPlayerListFooter(Report.helpText());
    }

    @Override
    protected @NotNull List<Listener> eventHandlers() {
        return List.of(new TablistListener());
    }
}
